Undefined Reference to `XYZ()' AKA Unresolved Symbols
-----------------------------------------------------

When building a program, the last step is to link together
all of the object (.o) files necessary to make a complete
program. The linker is used to complete this step. The linker
adds system library code that connects the program
with the operating system. That system library code calls
the function named `main()` as the starting point of the 
executable program. This makes having `main()` in your program
a requirement.

A linker command looks like this:

    g++ -std=c++11 -o desired_executable_program_name object_file.o another_object_file.o ... 
    
The `-o desired_executable_program_name` part must be given together,
and tells the linker what to name the program. The rest of the line
tells the linker which object files contain the complete set of
definitions to make a complete file.

The linker makes sure that every function *call* (e.g. `fred()`) in
the object files can be matched up with a single function definition
(e.g. `void fred() { std::cout << "Fred Flintstone" << std::endl; }`)
from the object files.

If the linker doesn't find any available definitions, it will display
an error message that there are unresolved symbols, or undefined symbols,
or something to that end.

If the linker finds too many (more than 1) for a particular definition,
it will display an error message that there are multiple definitions
of the function.

The code example available here tries to build `program3`.
This build will end up with an `undefined reference` error.

Cause of Undefined Reference Error
-----------------------------------------

`program3` has a `Makefile` entry that looks like this:

    program3: program3.o fred.o
        g++ -std=c++11 -o program3 program3.o fred.o
        
`program3.cpp` contains the definition of `main()`, which calls `barney()`.

`fred.cpp` contains a definition of `fred()`.

`barney.cpp` contains a definition of `barney()`, but also a definition of `fred()`.

So, every function that needs a definition has been defined in a .o file.
The linker command, is told the names of `program3.o` and `fred.o`. This is the
error message produced:

    g++ -std=c++11 -o program3 program3.o fred.o
    /usr/bin/ld: program3.o: in function `main':
    program3.cpp:(.text+0x9): undefined reference to `barney()'
    collect2: error: ld returned 1 exit status

What is the cause of the error? `barney()` has been defined in `barney.cpp`,
but `barney.o` was not given to the linker command.

What is the solution?


